@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2008-2023, University of Amsterdam
+    Copyright (c)  2008-2024, University of Amsterdam
 			      VU University Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -60,7 +60,7 @@ extern "C" {
 /* PLVERSION_TAG: a string, normally "", but for example "rc1" */
 
 #ifndef PLVERSION
-#define PLVERSION 90117
+#define PLVERSION 90303
 #endif
 #ifndef PLVERSION_TAG
 #define PLVERSION_TAG ""
@@ -78,7 +78,7 @@ numbers are atomic type.
 #define PL_FLI_VERSION      2		/* PL_*() functions */
 #define	PL_REC_VERSION      3		/* PL_record_external(), fastrw */
 #define PL_QLF_LOADVERSION 68		/* load all versions later >= X */
-#define PL_QLF_VERSION     68		/* save version number */
+#define PL_QLF_VERSION     69		/* save version number */
 
 
 		 /*******************************
@@ -480,7 +480,7 @@ PL_EXPORT(int)		PL_assert(term_t term, module_t m, int flags);
 		 *******************************/
 
 			/* Creating and destroying term-refs */
-PL_EXPORT(term_t)	PL_new_term_refs(int n);
+PL_EXPORT(term_t)	PL_new_term_refs(size_t n);
 PL_EXPORT(term_t)	PL_new_term_ref(void);
 PL_EXPORT(term_t)	PL_copy_term_ref(term_t from);
 PL_EXPORT(void)		PL_reset_term_refs(term_t r);
@@ -520,8 +520,7 @@ PL_EXPORT(int)		PL_get_atom(term_t t, atom_t *a) WUNUSED;
 PL_EXPORT(int)		PL_get_bool(term_t t, int *value) WUNUSED;
 PL_EXPORT(int)		PL_get_atom_chars(term_t t, char **a) WUNUSED;
 #define PL_get_string_chars(t, s, l) PL_get_string(t,s,l)
-					/* PL_get_string() is deprecated */
-PL_EXPORT(int)		PL_get_string(term_t t, char **s, size_t *len) WUNUSED;
+PL_EXPORT(int)		PL_get_string(term_t t, char **s, size_t *len) WUNUSED; /* WDEPRECATED */
 PL_EXPORT(int)		PL_get_chars(term_t t, char **s, unsigned int flags) WUNUSED;
 PL_EXPORT(int)		PL_get_list_chars(term_t l, char **s,
 					  unsigned int flags) WUNUSED;
@@ -683,6 +682,13 @@ PL_EXPORT(int)		PL_is_attvar(term_t t);
 PL_EXPORT(int)		PL_get_attr(term_t v, term_t a);
 
 
+		 /*******************************
+		 *           TABLING            *
+		 *******************************/
+
+PL_EXPORT(int)		PL_get_delay_list(term_t l);
+
+
 		 /*******************************
 		 *	      ERRORS		*
 		 *******************************/
@@ -870,6 +876,7 @@ PL_EXPORT(int)		PL_same_compound(term_t t1, term_t t2);
 PL_EXPORT(int)		PL_warning(const char *fmt, ...) WPRINTF12;
 PL_EXPORT(int)		PL_warningX(const char *fmt, ...);
 PL_EXPORT(void)		PL_fatal_error(const char *fmt, ...) WPRINTF12;
+PL_EXPORT(void)		PL_system_error(const char *fmt, ...) WPRINTF12;
 
 		 /*******************************
 		 *      RECORDED DATABASE	*
@@ -1283,6 +1290,7 @@ PL_EXPORT(intptr_t)	PL_query(int);	/* get information from Prolog */
 
 #define PL_THREAD_NO_DEBUG	0x01	/* Start thread in nodebug mode */
 #define PL_THREAD_NOT_DETACHED	0x02	/* Allow Prolog to join */
+#define PL_THREAD_CUR_STREAMS   0x04	/* Copy current{input,output} */
 
 typedef enum
 { PL_THREAD_CANCEL_FAILED = FALSE,	/* failed to cancel; try abort */
