@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2023, University of Amsterdam
+    Copyright (c)  2011-2024, University of Amsterdam
 			      VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -45,6 +45,7 @@
 #include "pl-prologflag.h"
 #include "pl-utf8.h"
 #include "pl-ctype.h"
+#include "pl-funct.h"
 #include "../pl-arith.h"
 #include "../pl-tabling.h"
 #include "../pl-fli.h"
@@ -112,9 +113,14 @@ static void setTZPrologFlag(void);
 static void setVersionPrologFlag(void);
 static void initPrologFlagTable(void);
 
+typedef struct oneof
+{ size_t	count;
+  int		references;
+  atom_t       *values;
+} oneof;
 
 typedef struct _prolog_flag
-{ short		flags;			/* Type | Flags */
+{ unsigned short flags;			/* Type | Flags */
   short		index;			/* index in LD->prolog_flag.mask */
   union
   { atom_t	a;			/* value as atom */
@@ -122,8 +128,18 @@ typedef struct _prolog_flag
     double	f;			/* value as float */
     record_t	t;			/* value as term */
   } value;
+  oneof        *oneof;
 } prolog_flag;
 
+#define unify_prolog_flag_value(m, key, f, val) \
+  LDFUNC(unify_prolog_flag_value, m, key, f, val)
+
+static int unify_prolog_flag_value(DECL_LD Module m, atom_t key, prolog_flag *f, term_t val);
+static int unify_prolog_flag_type(prolog_flag *f, term_t type);
+static int set_flag_type(prolog_flag *f, int flags);
+
+#define FF_WARN_NOT_ACCESSED 0x0100
+#define FF_ACCESSED          0x0200
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 C-interface for defining Prolog  flags.  Depending   on  the  type,  the
@@ -137,6 +153,91 @@ following arguments are to be provided:
     FT_TERM	a term
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+static void
+free_oneof(oneof *of)
+{ for(size_t i=0; i<of->count; i++)
+    PL_unregister_atom(of->values[i]);
+  freeHeap(of->values, of->count * sizeof(*of->values));
+  freeHeap(of, sizeof(of));
+}
+
+static oneof *
+create_oneof(term_t atoms)
+{ term_t tail = PL_copy_term_ref(atoms);
+  term_t head = PL_new_term_ref();
+  tmp_buffer b;
+
+  initBuffer(&b);
+  while(PL_get_list(tail, head, tail))
+  { atom_t a;
+
+    if ( PL_get_atom_ex(head, &a) )
+    { addBuffer(&b, a, atom_t);
+    } else
+    { error:
+      discardBuffer(&b);
+      return NULL;
+    }
+  }
+  if ( !PL_get_nil(tail) )
+    goto error;
+
+  oneof *of = allocHeapOrHalt(sizeof(*of));
+  of->count = entriesBuffer(&b, atom_t);
+  of->references = 1;
+  of->values = allocHeapOrHalt(of->count*sizeof(atom_t));
+  for(size_t i=0; i<of->count; i++)
+  { of->values[i] = fetchBuffer(&b, i, atom_t);
+    PL_register_atom(of->values[i]);
+  }
+
+  return of;
+}
+
+
+static int
+put_oneof(term_t t, const oneof *of)
+{ term_t h = PL_new_term_ref();
+
+  PL_put_nil(t);
+  for(ssize_t i=(ssize_t)of->count-1; i>=0; i--)
+  { if ( !PL_put_atom(h, of->values[i]) ||
+	 !PL_cons_list(t, h, t) )
+      return FALSE;
+  }
+
+  return PL_cons_functor(t, FUNCTOR_oneof1, t);
+}
+
+
+
+static int
+check_oneof_flag(const oneof *of, atom_t a, int error)
+{ if ( of )
+  { for(size_t i=0; i<of->count; i++)
+    { if ( of->values[i] == a )
+	return TRUE;
+    }
+
+    if ( error )
+    { term_t dom, ex;
+
+      return ( (dom=PL_new_term_ref()) &&
+	       (ex=PL_new_term_ref()) &&
+	       put_oneof(dom, of) &&
+	       PL_unify_term(ex, PL_FUNCTOR, FUNCTOR_error2,
+				   PL_FUNCTOR, FUNCTOR_domain_error2,
+				     PL_TERM, dom,
+				     PL_ATOM, a,
+				   PL_VARIABLE) &&
+	       PL_raise_exception(ex) );
+    } else
+      return FALSE;
+  }
+
+  return TRUE;
+}
+
 void
 setPrologFlag(const char *name, int flags, ...)
 { GET_LD
@@ -159,6 +260,7 @@ setPrologFlag(const char *name, int flags, ...)
   { f = allocHeapOrHalt(sizeof(*f));
     f->index = 0;
     f->flags = flags;
+    f->oneof = NULL;
     addNewHTable(GD->prolog_flag.table, (void *)an, f);
     first_def = TRUE;
   }
@@ -230,8 +332,12 @@ setPrologFlag(const char *name, int flags, ...)
 
 
 static void
-freePrologFlag(prolog_flag *f)
-{ switch((f->flags & FT_MASK))
+clean_prolog_flag(prolog_flag *f)
+{ int type = (f->flags & FT_MASK);
+  oneof *of;
+
+  set_flag_type(f, FT_INTEGER);
+  switch(type)
   { case FT_TERM:
       PL_erase(f->value.t);
       break;
@@ -241,6 +347,17 @@ freePrologFlag(prolog_flag *f)
     default:
       ;
   }
+  memset(&f->value, 0, sizeof(f->value));
+
+  if ( (of=f->oneof) && --of->references == 0 )
+  { f->oneof = NULL;
+    free_oneof(of);
+  }
+}
+
+static void
+freePrologFlag(prolog_flag *f)
+{ clean_prolog_flag(f);
 
   freeHeap(f, sizeof(*f));
 }
@@ -252,6 +369,8 @@ copy_prolog_flag(const prolog_flag *f)
 { prolog_flag *copy = allocHeapOrHalt(sizeof(*copy));
 
   *copy = *f;
+  if ( f->oneof )
+    f->oneof->references++;
   switch((f->flags & FT_MASK))
   { case FT_TERM:
       copy->value.t = PL_duplicate_record(f->value.t);
@@ -596,6 +715,22 @@ propagateAutoload(DECL_LD term_t val)
 }
 
 
+static void
+accessed_prolog_flag(prolog_flag *f, atom_t name, int local)
+{ if ( true(f, FF_WARN_NOT_ACCESSED) &&
+       false(f, FF_ACCESSED) )
+  { set(f, FF_ACCESSED);
+
+    if ( local )
+    { prolog_flag *fg;
+
+      if ( (fg = lookupHTable(GD->prolog_flag.table, (void *)name)) )
+	set(fg, FF_ACCESSED);
+    }
+  }
+}
+
+
 #if O_XOS
 typedef struct access_id
 { char *name;
@@ -640,9 +775,108 @@ get_win_file_access_check(void)
 }
 #endif
 
-#define set_prolog_flag_unlocked(m, k, value, flags) LDFUNC(set_prolog_flag_unlocked, m, k, value, flags)
-static word
-set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, int flags)
+#define FF_COPY_FLAGS (FF_READONLY|FF_WARN_NOT_ACCESSED)
+#define PSEUDO_FLAG ((prolog_flag*)TRUE)
+
+static int
+set_flag_type(prolog_flag *f, int flags)
+{ f->flags = (f->flags&~FT_MASK) | (flags&FT_MASK);
+  return TRUE;
+}
+
+static int
+set_flag_atom(prolog_flag *f, atom_t a)
+{ if ( f->value.a != a )
+  { PL_unregister_atom(f->value.a);
+    PL_register_atom(a);
+    f->value.a = a;
+  }
+
+  return TRUE;
+}
+
+
+static int
+keep_flag(atom_t k, prolog_flag *f, int flags, oneof *of, term_t value)
+{ if ( (flags&FF_KEEP) )
+  { if ( of )
+    { if ( check_oneof_flag(of, f->value.a, FALSE) )
+	return TRUE;
+    } else if ( (flags&FT_MASK) == (f->flags&FT_MASK) )
+    { return TRUE;
+    } else
+    { int ftype = (f->flags&FT_MASK);
+
+      switch(flags&FT_MASK)	/* target type */
+      { case FT_FROM_VALUE:
+	  return TRUE;
+	case FT_ATOM:
+	  if ( ftype == FT_BOOL )
+	    return set_flag_type(f, flags);
+	  break;
+	case FT_BOOL:
+	  if ( ftype == FT_ATOM )
+	  { int bv = atom_to_bool(f->value.a);
+	    if ( bv >= 0 )
+	    { set_flag_type(f, flags);
+	      return set_flag_atom(f, bv ? ATOM_true : ATOM_false);
+	    }
+	  }
+	  break;
+	case FT_FLOAT:
+	  if ( ftype == FT_INTEGER )
+	  { f->value.f = (double)f->value.i;
+	    return set_flag_type(f, flags);
+	  }
+	  break;
+	case FT_TERM:
+	{ term_t value;
+
+	  if ( (value=PL_new_term_ref()) &&
+	       unify_prolog_flag_value(MODULE_user, k, f, value) &&
+	       (f->value.t=PL_record(value)) )
+	    return set_flag_type(f, flags);
+	  return -1;
+	}
+	default:
+	  break;
+      }
+    }
+
+    prolog_flag f2 = *f;
+    f2.oneof = of;
+    f2.flags = flags;
+    term_t type;
+    term_t preset;
+
+    if ( !((type = PL_new_term_ref()) &&
+	   (preset = PL_new_term_ref()) &&
+	   unify_prolog_flag_type(&f2, type) &&
+	   unify_prolog_flag_value(MODULE_user, k, f, preset) &&
+	   printMessage(ATOM_warning,
+			  PL_FUNCTOR_CHARS, "prolog_flag_invalid_preset", 4,
+			    PL_ATOM, k,
+			    PL_TERM, preset,
+			    PL_TERM, type,
+			PL_TERM, value) ) )
+      return -1;
+
+    clean_prolog_flag(f);
+    set_flag_type(f, flags);
+    f->oneof = of;
+
+    return FALSE;
+  }
+
+  return FALSE;
+}
+
+
+#define set_prolog_flag_unlocked(m, k, value, flags, of) \
+	LDFUNC(set_prolog_flag_unlocked, m, k, value, flags, of)
+
+static prolog_flag *
+set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, int flags, oneof *of)
 { prolog_flag *f;
   int rval = TRUE;
 
@@ -650,26 +884,36 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, int flags)
 #ifdef O_PLMT
   if ( LD->prolog_flag.table &&
        (f = lookupHTable(LD->prolog_flag.table, (void *)k)) )
-  { if ( flags & FF_KEEP )
-      return TRUE;
+  { int rc;
+    accessed_prolog_flag(f, k, TRUE);
+    if ( (rc=keep_flag(k, f, flags, of, value)) == TRUE )
+      return f;
+    if ( rc == -1 )
+      return NULL;
   } else
 #endif
   if ( (f = lookupHTable(GD->prolog_flag.table, (void *)k)) )
-  { if ( flags & FF_KEEP )
-      return TRUE;
+  { int rc;
+    accessed_prolog_flag(f, k, FALSE);
+    if ( (rc=keep_flag(k, f, flags, of, value)) == TRUE )
+      return f;
+    if ( rc == -1 )
+      return NULL;
     if ( (f->flags&FF_READONLY) && !(flags&FF_FORCE) )
     { term_t key;
 
-      return ( (key = PL_new_term_ref()) &&
-	       PL_put_atom(key, k) &&
-	       PL_error(NULL, 0, NULL, ERR_PERMISSION,
-			ATOM_modify, ATOM_flag, key) );
+      if ( (key = PL_new_term_ref()) &&
+	   PL_put_atom(key, k) &&
+	   PL_error(NULL, 0, NULL, ERR_PERMISSION,
+		    ATOM_modify, ATOM_flag, key) )
+	return NULL;
+      return NULL;
     }
 
     if ( tbl_is_restraint_flag(k) )
-      return tbl_set_restraint_flag(value, k);
+      return tbl_set_restraint_flag(value, k) ? PSEUDO_FLAG : NULL;
     if ( is_arith_flag(k) )
-      return set_arith_flag(value, k);
+      return set_arith_flag(value, k) ? PSEUDO_FLAG : NULL;
 
 #ifdef O_PLMT
     if ( GD->statistics.threads_created > 1 )
@@ -698,17 +942,21 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, int flags)
     PL_register_atom(k);
     f = allocHeapOrHalt(sizeof(*f));
     f->index = 0;
+    f->oneof = NULL;
 
     switch( (flags & FT_MASK) )
     { case FT_FROM_VALUE:
       { if ( PL_get_atom(value, &a) )
-	{ if ( a == ATOM_true || a == ATOM_false ||
-	       a == ATOM_on || a == ATOM_off )
+	{ int bv = atom_to_bool(a);
+
+	  if ( bv >= 0 )
+	  { f->value.a = bv ? ATOM_true : ATOM_false;
 	    f->flags = FT_BOOL;
-	  else
+	  } else
+	  { f->value.a = a;
 	    f->flags = FT_ATOM;
-	  f->value.a = a;
-	  PL_register_atom(a);
+	  }
+	  PL_register_atom(f->value.a);
 	} else if ( PL_get_int64(value, &i) )
 	{ f->flags = FT_INTEGER;
 	  f->value.i = i;
@@ -766,8 +1014,7 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, int flags)
 	break;
     }
 
-    if ( (flags & FF_READONLY) )
-      f->flags |= FF_READONLY;
+    f->flags |= (flags&FF_COPY_FLAGS);
 
     addNewHTable(GD->prolog_flag.table, (void *)k, f);
     if ( !(lookupHTable(GD->prolog_flag.table, (void *)k) == f) )
@@ -775,7 +1022,7 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, int flags)
       Sdprintf("OOPS; failed to set Prolog flag!?\n");
     }
 
-    succeed;
+    return f;
   } else
   { atom_t how;
 
@@ -783,10 +1030,12 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, int flags)
     { if ( how == ATOM_error )
       { term_t key;
 
-	return ( (key = PL_new_term_ref()) &&
-		 PL_put_atom(key, k) &&
-		 PL_error(NULL, 0, NULL, ERR_EXISTENCE,
-			  ATOM_prolog_flag, key) );
+	if ( (key = PL_new_term_ref()) &&
+	     PL_put_atom(key, k) &&
+	     PL_error(NULL, 0, NULL, ERR_EXISTENCE,
+		      ATOM_prolog_flag, key) )
+	  return NULL;
+	return NULL;
       } else if ( how == ATOM_warning )
 	Sdprintf("WARNING: Flag %s: new Prolog flags must be created using "
 		 "create_prolog_flag/3\n", stringAtom(k));
@@ -847,7 +1096,7 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, int flags)
 
 	  if ( (ex = PL_new_term_ref()) &&
 	       PL_put_atom(ex, ATOM_protect_static_code) )
-	    return PL_permission_error("set", "prolog_flag", ex);
+	    return PL_permission_error("set", "prolog_flag", ex),NULL;
 	  return FALSE;
 	}
       }
@@ -868,6 +1117,9 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, int flags)
       if ( !PL_get_atom_ex(value, &a) )
 	return FALSE;
 
+      if ( !check_oneof_flag(f->oneof, a, TRUE) )
+	return FALSE;
+
       if ( k == ATOM_double_quotes )
       { rval = setDoubleQuotes(a, &m->flags);
       } else if ( k == ATOM_back_quotes )
@@ -965,16 +1217,16 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, int flags)
       assert(0);
   }
 
-  return rval;
+  return rval ? f : NULL;
 }
 
 
-int
-set_prolog_flag(term_t key, term_t value, int flags)
+static prolog_flag *
+set_prolog_flag_ptr(term_t key, term_t value, int flags, oneof *of)
 { GET_LD
   atom_t k;
   Module m = MODULE_parse;
-  int rc;
+  prolog_flag *f;
 
   if ( !PL_strip_module(key, &m, key) ||
        !PL_get_atom_ex(key, &k) )
@@ -983,13 +1235,18 @@ set_prolog_flag(term_t key, term_t value, int flags)
   if ( k == ATOM_autoload && !propagateAutoload(value) )
     return FALSE;
   if ( k == ATOM_threads )
-    return set_prolog_flag_unlocked(m, k, value, flags);
+    return set_prolog_flag_unlocked(m, k, value, flags, of);
 
   PL_LOCK(L_PLFLAG);
-  rc = set_prolog_flag_unlocked(m, k, value, flags);
+  f = set_prolog_flag_unlocked(m, k, value, flags, of);
   PL_UNLOCK(L_PLFLAG);
 
-  return rc;
+  return f;
+}
+
+int
+set_prolog_flag(term_t key, term_t value, int flags)
+{ return !!set_prolog_flag_ptr(key, value, flags, NULL);
 }
 
 /** set_prolog_flag(+Key, +Value) is det.
@@ -1005,42 +1262,50 @@ PRED_IMPL("set_prolog_flag", 2, set_prolog_flag, PL_FA_ISO)
 */
 
 static const PL_option_t prolog_flag_options[] =
-{ { ATOM_type,   OPT_ATOM },
-  { ATOM_access, OPT_ATOM },
-  { ATOM_keep,   OPT_BOOL },
-  { NULL_ATOM,   0 }
+{ { ATOM_type,              OPT_TERM },
+  { ATOM_access,            OPT_ATOM },
+  { ATOM_keep,              OPT_BOOL },
+  { ATOM_warn_not_accessed, OPT_BOOL },
+  { NULL_ATOM,              0 }
 };
 
 static
 PRED_IMPL("create_prolog_flag", 3, create_prolog_flag, PL_FA_ISO)
 { PRED_LD
   int flags = 0;
-  atom_t type = 0;
+  term_t type = 0;
   atom_t access = ATOM_read_write;
   int keep = FALSE;
+  int warn_not_accessed = FALSE;
+  oneof *of = NULL;
+  atom_t a;
 
   if ( !PL_scan_options(A3, 0, "prolog_flag_option", prolog_flag_options,
-		     &type, &access, &keep) )
+			&type, &access, &keep, &warn_not_accessed) )
     return FALSE;
 
   if ( type == 0 )
-    flags |= FT_FROM_VALUE;
-  else if ( type == ATOM_boolean )
-    flags |= FT_BOOL;
-  else if ( type == ATOM_integer )
-    flags |= FT_INTEGER;
-  else if ( type == ATOM_float )
-    flags |= FT_FLOAT;
-  else if ( type == ATOM_atom )
+  { flags |= FT_FROM_VALUE;
+  } else if ( PL_get_atom(type, &a) )
+  { if ( a == ATOM_boolean )
+      flags |= FT_BOOL;
+    else if ( a == ATOM_integer )
+      flags |= FT_INTEGER;
+    else if ( a == ATOM_float )
+      flags |= FT_FLOAT;
+    else if ( a == ATOM_atom )
+      flags |= FT_ATOM;
+    else if ( a == ATOM_term )
+      flags |= FT_TERM;
+    else
+      return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_prolog_flag_type, type);
+  } else if ( PL_is_functor(type, FUNCTOR_oneof1) )
+  { _PL_get_arg(1, type, type);
+    if ( !(of=create_oneof(type)) )
+      return FALSE;
     flags |= FT_ATOM;
-  else if ( type == ATOM_term )
-    flags |= FT_TERM;
-  else
-  { term_t a = PL_new_term_ref();
-    PL_put_atom(a, type);
-
-    return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_prolog_flag_type, a);
-  }
+  } else
+    return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_prolog_flag_type, type);
 
   if ( access == ATOM_read_only )
     flags |= FF_READONLY;
@@ -1053,7 +1318,19 @@ PRED_IMPL("create_prolog_flag", 3, create_prolog_flag, PL_FA_ISO)
   if ( keep )
     flags |= FF_KEEP;
 
-  return set_prolog_flag(A1, A2, flags);
+  if ( warn_not_accessed )
+    flags |= FF_WARN_NOT_ACCESSED;
+
+  prolog_flag *f = set_prolog_flag_ptr(A1, A2, flags, of);
+
+  if ( of )
+  { if ( f && f != PSEUDO_FLAG )
+      f->oneof = of;
+    else
+      free_oneof(of);
+  }
+
+  return !!f;
 }
 
 
@@ -1114,13 +1391,12 @@ PL_current_prolog_flag(atom_t name, int type, void *value)
   return FALSE;
 }
 
-
+#define unify_prolog_flag_value(m, key, f, val) \
+  LDFUNC(unify_prolog_flag_value, m, key, f, val)
 
 static int
-unify_prolog_flag_value(Module m, atom_t key, prolog_flag *f, term_t val)
-{ GET_LD
-
-  if ( key == ATOM_character_escapes )
+unify_prolog_flag_value(DECL_LD Module m, atom_t key, prolog_flag *f, term_t val)
+{ if ( key == ATOM_character_escapes )
   { return PL_unify_bool(val, true(m, M_CHARESCAPE));
   } else if ( key == ATOM_var_prefix )
   { return PL_unify_bool(val, true(m, M_VARPREFIX));
@@ -1250,6 +1526,13 @@ unify_prolog_flag_type(prolog_flag *f, term_t type)
       a = ATOM_boolean;
       break;
     case FT_ATOM:
+      if ( f->oneof )
+      { term_t t;
+
+	return ( (t=PL_new_term_ref()) &&
+		 put_oneof(t, f->oneof) &&
+		 PL_unify(t, type) );
+      }
       a = ATOM_atom;
       break;
     case FT_INTEGER:
@@ -1278,11 +1561,10 @@ typedef struct
 } prolog_flag_enum;
 
 word
-pl_prolog_flag5(term_t key, term_t value,
-	    word scope, word access, word type,
-	    control_t h)
-{ GET_LD
-  prolog_flag_enum *e;
+pl_prolog_flag5(DECL_LD term_t key, term_t value,
+		word scope, word access, word type,
+		control_t h)
+{ prolog_flag_enum *e;
   fid_t fid;
   Module module;
 
@@ -1300,13 +1582,17 @@ pl_prolog_flag5(term_t key, term_t value,
 #ifdef O_PLMT
 	if ( LD->prolog_flag.table &&
 	     (f = lookupHTable(LD->prolog_flag.table, (void *)k)) )
-	  return unify_prolog_flag_value(module, k, f, value);
+	{ accessed_prolog_flag(f, k, TRUE);
+	  return ( unify_prolog_flag_value(module, k, f, value) &&
+		   (!access || unify_prolog_flag_access(f, access)) &&
+		   (!type   || unify_prolog_flag_type(f, type)) );
+	}
 #endif
 	if ( (f = lookupHTable(GD->prolog_flag.table, (void *)k)) )
-	{ if ( unify_prolog_flag_value(module, k, f, value) &&
-	       (!access || unify_prolog_flag_access(f, access)) &&
-	       (!type   || unify_prolog_flag_type(f, type)) )
-	    succeed;
+	{ accessed_prolog_flag(f, k, FALSE);
+	  return ( unify_prolog_flag_value(module, k, f, value) &&
+		   (!access || unify_prolog_flag_access(f, access)) &&
+		   (!type   || unify_prolog_flag_type(f, type)) );
 	}
 
 	fail;
@@ -1395,10 +1681,18 @@ pl_prolog_flag5(term_t key, term_t value,
   fail;
 }
 
+static
+PRED_IMPL("$current_prolog_flag", 5, dcurrent_prolog_flag, PL_FA_NONDETERMINISTIC)
+{ PRED_LD
 
-foreign_t
-pl_prolog_flag(term_t name, term_t value, control_t h)
-{ return pl_prolog_flag5(name, value, 0, 0, 0, h);
+  return pl_prolog_flag5(A1, A2, A3, A4, A5, PL__ctx);
+}
+
+static
+PRED_IMPL("current_prolog_flag", 2, current_prolog_flag, PL_FA_ISO|PL_FA_NONDETERMINISTIC)
+{ PRED_LD
+
+  return pl_prolog_flag5(A1, A2, 0, 0, 0, PL__ctx);
 }
 
 
@@ -1522,7 +1816,6 @@ initPrologFlags(void)
 #endif
   setPrologFlag("optimise", FT_BOOL, GD->cmdline.optimise, PLFLAG_OPTIMISE);
   setPrologFlag("optimise_unify", FT_BOOL, TRUE, PLFLAG_OPTIMISE_UNIFY);
-  setPrologFlag("optimise_debug", FT_ATOM, "default", 0);
   setPrologFlag("generate_debug_info", FT_BOOL,
 		truePrologFlag(PLFLAG_DEBUGINFO), PLFLAG_DEBUGINFO);
   setPrologFlag("protect_static_code", FT_BOOL, FALSE,
@@ -1605,6 +1898,7 @@ initPrologFlags(void)
   setPrologFlag("min_tagged_integer", FT_INTEGER|FF_READONLY, (intptr_t)PLMINTAGGEDINT);
 #ifdef O_BIGNUM
   setPrologFlag("bounded",	      FT_BOOL|FF_READONLY,	   FALSE, 0);
+  setPrologFlag("rationals",	      FT_BOOL|FF_READONLY,	   TRUE, 0);
   setPrologFlag("prefer_rationals", FT_BOOL, O_PREFER_RATIONALS, PLFLAG_RATIONAL);
   setPrologFlag("rational_syntax",  FT_ATOM,
 		O_RATIONAL_SYNTAX == RAT_NATURAL ? "natural" :
@@ -1681,6 +1975,17 @@ initPrologFlags(void)
   setPrologFlag("android_api",FT_INTEGER|FF_READONLY, (intptr_t)__ANDROID_API__);
 # endif
 #endif
+#ifdef __CONDA__
+  setPrologFlag("conda", FT_BOOL|FF_READONLY, TRUE, 0);
+#endif
+#endif
+
+#if __WINDOWS__
+  #if __CONDA__ || MSYS2
+  setPrologFlag("xdg", FT_BOOL, TRUE, 0);
+  #endif
+#else
+  setPrologFlag("xdg", FT_BOOL|FF_READONLY, TRUE, 0);
 #endif
 
   setPrologFlag("encoding", FT_ATOM,
@@ -1710,6 +2015,11 @@ initPrologFlags(void)
 #ifdef POSIX_SHELL
   setPrologFlag("posix_shell", FT_ATOM, POSIX_SHELL);
 #endif
+#ifdef __WINDOWS__
+  setPrologFlag("path_sep", FT_ATOM, ";");
+#else
+  setPrologFlag("path_sep", FT_ATOM, ":");
+#endif
 
   setPrologFlag("table_incremental", FT_BOOL, FALSE, PLFLAG_TABLE_INCREMENTAL);
   setPrologFlag("table_subsumptive", FT_BOOL, FALSE, 0);
@@ -1789,6 +2099,7 @@ setVersionPrologFlag(void)
   int patch = (PLVERSION%100);
 
   PL_put_nil(o);
+  initFunctors();
 
 #ifdef PLVERSION_TAG
   { const char *tag = PLVERSION_TAG;
@@ -1854,6 +2165,51 @@ setABIVersionPrologFlag(void)
 }
 
 
+int
+checkPrologFlagsAccess(void)
+{ int rc = TRUE;
+
+  if ( GD->prolog_flag.table )
+  { if ( HAS_LD )
+    { GET_LD
+      fid_t fid = PL_open_foreign_frame();
+
+      if ( fid )
+      { term_t list = PL_new_term_ref();
+	term_t tail = PL_copy_term_ref(list);
+	term_t head = PL_new_term_ref();
+	int found = 0;
+
+	FOR_TABLE(GD->prolog_flag.table, n, v)
+	{ atom_t name = (atom_t)n;
+	  prolog_flag *f = (prolog_flag*)v;
+
+	  if ( true(f, FF_WARN_NOT_ACCESSED) &&
+	       false(f, FF_ACCESSED) )
+	  { found++;
+	    if ( !PL_unify_list(tail, head, tail) ||
+		 !PL_unify_atom(head, name) )
+	    { rc = FALSE;
+	      break;
+	    }
+	  }
+	}
+	if ( found && rc )
+	{ rc = ( PL_unify_nil(tail) &&
+		 printMessage(ATOM_warning,
+			      PL_FUNCTOR_CHARS, "not_accessed_flags", 1,
+				PL_TERM, list) );
+	}
+
+	PL_discard_foreign_frame(fid);
+      }
+    }
+  }
+
+  return rc;
+}
+
+
 void
 cleanupPrologFlags(void)
 { if ( GD->prolog_flag.table )
@@ -1875,7 +2231,11 @@ cleanupPrologFlags(void)
 		 *      PUBLISH PREDICATES	*
 		 *******************************/
 
+#define NDET PL_FA_NONDETERMINISTIC
+
 BeginPredDefs(prologflag)
-  PRED_DEF("set_prolog_flag",    2, set_prolog_flag,    PL_FA_ISO)
-  PRED_DEF("create_prolog_flag", 3, create_prolog_flag, 0)
+  PRED_DEF("set_prolog_flag",      2, set_prolog_flag,      PL_FA_ISO)
+  PRED_DEF("create_prolog_flag",   3, create_prolog_flag,   0)
+  PRED_DEF("current_prolog_flag",  2, current_prolog_flag,  PL_FA_ISO|NDET)
+  PRED_DEF("$current_prolog_flag", 5, dcurrent_prolog_flag, NDET)
 EndPredDefs
