@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2022, University of Amsterdam
+    Copyright (c)  2011-2024, University of Amsterdam
 			      VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -133,6 +133,7 @@ static void	run_close_hooks(IOSTREAM *s);
 static int	S__removebuf(IOSTREAM *s);
 static int	S__seterror(IOSTREAM *s);
        void	unallocStream(IOSTREAM *s);
+static int	S__fileno(IOSTREAM *s);
 
 static IOSTREAM *	Sopen_buffer(IOSTREAM *s, char *buf, size_t size);
 static void		Sclose_buffer(IOSTREAM *s);
@@ -381,6 +382,23 @@ Sunlock(IOSTREAM *s)
 }
 
 
+IOSTREAM *
+Sacquire(IOSTREAM *s)
+{ Sreference(s);
+  return s;
+}
+
+int
+Srelease(IOSTREAM *s)
+{ if ( Sunreference(s) == 0 && s->erased )
+  { unallocStream(s);
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+
 		 /*******************************
 		 *		TIMEOUT		*
 		 *******************************/
@@ -3330,9 +3348,11 @@ Snew(void *handle, int flags, IOFUNCTIONS *functions)
   }
 #endif
 
-{ int fd;
-  if ( (fd = Sfileno(s)) >= 0 )
-  { if ( isatty(fd) )
+  s->fileno = S__fileno(s);
+  if ( s->fileno >= 0 )
+  { int fd = (int)s->fileno;
+
+    if ( isatty(fd) )
       s->flags |= SIO_ISATTY;
 
 #if defined(F_SETFD)
@@ -3342,7 +3362,6 @@ Snew(void *handle, int flags, IOFUNCTIONS *functions)
 			 HANDLE_FLAG_INHERIT, 0);
 #endif
   }
-}
 
 #ifdef O_LOCALE
   initStreamLocale(s);
@@ -3559,6 +3578,32 @@ Sopen_iri_or_file(const char *path, const char *how)
   return s;
 }
 
+#ifdef __WINDOWS__
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+[not sure] It seems the CRT library of Windows can operate in two modes:
+one where the CRT state is  shared   between  the application and shared
+objects loaded into it, and one where this  is not the case. The default
+Windows build uses the shared view  and   the  Conda build the separated
+view.  This means that CRT handles cannot cross a component boundary.
+
+The clib/process.c component needs to turn an   I/O HANDLE into a Prolog
+stream.  As  Prolog  streams  use   the    CRT   handle,  we  must  call
+_open_osfhandle in the SWI-Prolog core.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+int
+Swin_open_osfhandle(HANDLE h, int flags)
+{ return _open_osfhandle((intptr_t)h, flags);
+}
+
+IOSTREAM *
+Swin_open_handle(HANDLE h, const char *mode)
+{ int fd = Swin_open_osfhandle(h, 0);
+
+  return Sfdopen(fd, mode);
+}
+#endif
+
 
 IOSTREAM *
 Sfdopen(int fd, const char *type)
@@ -3593,20 +3638,13 @@ Sfdopen(int fd, const char *type)
 /* MT: as long as s is valid, this should be ok
 */
 
-int
-Sfileno(IOSTREAM *s)
-{ int n;
-
-  if ( s->magic != SIO_MAGIC )
-  { errno = EINVAL;
-    return -1;
-  }
-
-  if ( s->flags & SIO_FILE )
-  { intptr_t h = (intptr_t)s->handle;
-    n = (int)h;
+static int
+S__fileno(IOSTREAM *s)
+{ if ( (s->flags & SIO_FILE) )
+  { return (int)(intptr_t)s->handle;
   } else
   { IOFUNCTIONS *funcs = s->functions;
+    int n;
 
     if ( s->magic == SIO_MAGIC &&
 	 funcs->control &&
@@ -3618,12 +3656,45 @@ Sfileno(IOSTREAM *s)
     errno = EINVAL;
     return -1;
   }
+}
 
-  return n;
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Sfileno() returns  the cached file  no rather then  calling Scontrol()
+using  SIO_GETFILENO.   This  is  to  ensure  safe  stream_property(S,
+file_no(I)), which cannot lock the  stream.  The `fileno` field is set
+by Snew().   If the stream is  modified such that the  associated file
+handle  is   changed,  the  user   must  update  the   `fileno`  field
+accordingly.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+int
+Sfileno(IOSTREAM *s)
+{ if ( s->magic != SIO_MAGIC )
+  { errno = EINVAL;
+    return -1;
+  }
+
+  DEBUG(0, assert(s->fileno == S__fileno(s)));
+  if ( s->fileno >= 0 )
+    return (int)s->fileno;
+
+  return -1;
 }
 
 
 #ifdef __WINDOWS__
+
+HANDLE
+Swinhandle(IOSTREAM *s)
+{ int fd = Sfileno(s);
+
+  if ( fd >= 0 )
+    return (HANDLE)_get_osfhandle(fd);
+
+  return NULL;
+}
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 On  Windows,  type  SOCKET  is   an    unsigned   int   and  all  values
 [0..INVALID_SOCKET) are valid. It is  also   not  allowed  to run normal
@@ -4190,10 +4261,10 @@ Sclose_buffer(IOSTREAM *s)
 		 *******************************/
 
 #define STDIO(n, f) \
-	{ .bufp = NULL, .limitp = NULL, .buffer = NULL, .unbuffer = NULL, \
-	  .lastc = EOF, .magic = SIO_MAGIC, .flags = (f),		  \
-	  .handle = (void *)(n), .functions = &Sttyfunctions,		  \
-	  .timeout = -1, .encoding = ENC_ISO_LATIN_1			  \
+	{ .bufp = NULL, .limitp = NULL, .buffer = NULL, .unbuffer = NULL,  \
+	  .lastc = EOF, .magic = SIO_MAGIC, .flags = (f),		   \
+	  .handle = (void *)(n), .fileno = n, .functions = &Sttyfunctions, \
+	  .timeout = -1, .encoding = ENC_ISO_LATIN_1			   \
 	}
 
 #define SIO_STDIO (SIO_FILE|SIO_STATIC|SIO_NOCLOSE|SIO_ISATTY|SIO_TEXT)
