@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2022, University of Amsterdam
+    Copyright (c)  1985-2024, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -191,7 +191,7 @@ writeNumberVar(DECL_LD term_t t, write_options *options)
   if ( LD->var_names.numbervars_frame )
   { FliFrame fr = (FliFrame)valTermRef(LD->var_names.numbervars_frame);
 
-    assert(fr->magic == FLI_MAGIC);
+    FLI_ASSERT_VALID(fr);
     if ( false(options, PL_WRT_NUMBERVARS) &&
          fr->mark.globaltop > (Word)f )
       return FALSE;			/* older $VAR term */
@@ -1208,23 +1208,24 @@ writeMPZ(DECL_LD mpz_t mpz, write_options *options)
   char *buf;
   size_t sz = (mpz_sizeinbase(mpz, 2)*10)/3 + 10; /* log2(10)=3.322 */
   int rc;
+  AR_CTX;
 
   if ( sz <= sizeof(tmp) )
     buf = tmp;
-  else
-    buf = PL_malloc(sz);
+  else if ( !(buf = tmp_malloc(sz)) )
+    return PL_no_memory();
 
   /* mpz_get_str() can perform large intermediate allocations */
-  EXCEPTION_GUARDED({ LD->gmp.persistent++;
-		      mpz_get_str(buf, 10, mpz);
-		      LD->gmp.persistent--;
+  AR_BEGIN();
+  EXCEPTION_GUARDED({ mpz_get_str(buf, 10, mpz);
+                      rc = TRUE;
 		    },
-		    { LD->gmp.persistent--;
-		      rc = PL_rethrow();
-		    })
-  rc = PutToken(buf, options->out);
+		    { rc = FALSE;
+		    });
+  AR_END();
+  rc = rc && PutToken(buf, options->out);
   if ( buf != tmp )
-    PL_free(buf);
+    tmp_free(buf);
 
   return rc;
 }
@@ -2432,8 +2433,7 @@ PRED_IMPL("write_length", 3, write_length, 0)
     s->flags |= SIO_USERBUF;
 
     clearPrologFlagMask(PLFLAG_ISO);
-    pushOutputContext();
-    Scurout = s;
+    pushOutputContext(s);
     rc = pl_write_term3(0, A1, A3);
     popOutputContext();
     LD->prolog_flag.mask = oldmask;
