@@ -499,8 +499,8 @@ class Prolog
 	throw(`Prolog.predicate: illegal specification: ${name}`);
     }
 
-    const c_name   = allocateUTF8(name);
-    const c_module = allocateUTF8(module||"user");
+    const c_name   = stringToNewUTF8(name);
+    const c_module = stringToNewUTF8(module||"user");
 
     const pred = this.bindings.PL_predicate(c_name, arity, c_module);
 
@@ -1265,6 +1265,7 @@ class Prolog
 	      default:
 	      { let id = prolog.object_ids.get(data);
 
+		prolog.bindings.PL_put_variable(term);
 		if ( id === undefined )
 		{ id = prolog.next_object_id+1;
 		  rc = prolog.bindings.js_unify_obj(term, id);
